package masterelection

import (
	"fmt"
	"log"
	"math/rand"
	"os"
	"testing"
	"time"

	"git.autistici.org/ale/autoradio/util"
)

func init() {
	rand.Seed(time.Now().Unix())
}

func countMasters(nodes []*MasterElection) int {
	var masters int
	for _, n := range nodes {
		if n.IsMaster() {
			masters++
		}
	}
	return masters
}

func verifyMasterData(t *testing.T, nodes []*MasterElection) {
	tmp := make(map[string]struct{})
	for _, n := range nodes {
		if n != nil {
			tmp[n.GetMasterData()] = struct{}{}
		}
	}
	if len(tmp) != 1 {
		t.Errorf("master data propagation error: >1 values: %v", tmp)
	}
}

func TestMasterElection(t *testing.T) {
	//etcd := util.NewTestEtcdClient()
	etcd := util.NewTestEtcdClientWithLatency(20 * time.Millisecond)
	lockPath := "/master/election/test"

	n := 5
	var nodes []*MasterElection
	var stop []chan bool
	for i := 0; i < n; i++ {
		stopCh := make(chan bool)
		m := NewMasterElection(
			etcd,
			lockPath,
			fmt.Sprintf("%d", i),
			1,
			nil)
		m.Log = log.New(os.Stderr, fmt.Sprintf("node%d: masterelection: ", i+1), 0)
		go m.Run(stopCh)
		nodes = append(nodes, m)
		stop = append(stop, stopCh)
	}

	// Force master transitions by shutting down the nodes one by
	// one as the become the master, and verify that there is a
	// single master among the remaining ones.
	for i := 0; i < n; i++ {
		time.Sleep(100 * time.Millisecond)

		masterIdx := -1
		numMasters := 0
		for i, m := range nodes {
			if m == nil {
				continue
			}
			if m.IsMaster() {
				numMasters++
				masterIdx = i
			}
		}

		if numMasters != 1 {
			t.Fatalf("masters=%d (expected 1): %#v", numMasters, nodes)
		}
		verifyMasterData(t, nodes)

		close(stop[masterIdx])
		nodes[masterIdx] = nil
	}
}
